@@ -1,6 +1,6 @@
 /* connection_type.c - python interface to connection objects
  *
- * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
@@ -29,6 +29,7 @@
 #include "psycopg/connection.h"
 #include "psycopg/cursor.h"
 #include "psycopg/pqpath.h"
+#include "psycopg/conninfo.h"
 #include "psycopg/lobject.h"
 #include "psycopg/green.h"
 #include "psycopg/xid.h"
@@ -111,10 +112,10 @@ psyco_conn_cursor(connectionObject *self, PyObject *args, PyObject *kwargs)
         goto exit;
     }
 
-    if (0 != psyco_curs_withhold_set((cursorObject *)obj, withhold)) {
+    if (0 > curs_withhold_set((cursorObject *)obj, withhold)) {
         goto exit;
     }
-    if (0 != psyco_curs_scrollable_set((cursorObject *)obj, scrollable)) {
+    if (0 > curs_scrollable_set((cursorObject *)obj, scrollable)) {
         goto exit;
     }
 
@@ -137,7 +138,7 @@ exit:
 #define psyco_conn_close_doc "close() -- Close the connection."
 
 static PyObject *
-psyco_conn_close(connectionObject *self)
+psyco_conn_close(connectionObject *self, PyObject *dummy)
 {
     Dprintf("psyco_conn_close: closing connection at %p", self);
     conn_close(self);
@@ -152,7 +153,7 @@ psyco_conn_close(connectionObject *self)
 #define psyco_conn_commit_doc "commit() -- Commit all changes to database."
 
 static PyObject *
-psyco_conn_commit(connectionObject *self)
+psyco_conn_commit(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, commit);
@@ -171,7 +172,7 @@ psyco_conn_commit(connectionObject *self)
 "rollback() -- Roll back all changes done to database."
 
 static PyObject *
-psyco_conn_rollback(connectionObject *self)
+psyco_conn_rollback(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, rollback);
@@ -244,7 +245,7 @@ exit:
 "tpc_prepare() -- perform the first phase of a two-phase transaction."
 
 static PyObject *
-psyco_conn_tpc_prepare(connectionObject *self)
+psyco_conn_tpc_prepare(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, tpc_prepare);
@@ -387,7 +388,7 @@ psyco_conn_tpc_rollback(connectionObject *self, PyObject *args)
 "tpc_recover() -- returns a list of pending transaction IDs."
 
 static PyObject *
-psyco_conn_tpc_recover(connectionObject *self)
+psyco_conn_tpc_recover(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, tpc_recover);
@@ -402,7 +403,7 @@ psyco_conn_tpc_recover(connectionObject *self)
 "__enter__ -> self"
 
 static PyObject *
-psyco_conn_enter(connectionObject *self)
+psyco_conn_enter(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
 
@@ -477,7 +478,7 @@ _psyco_conn_parse_isolevel(PyObject *pyval)
 
     /* parse from the string -- this includes "default" */
     else {
-        if (!(pyval = psycopg_ensure_bytes(pyval))) {
+        if (!(pyval = psyco_ensure_bytes(pyval))) {
             goto exit;
         }
         for (level = 1; level <= 4; level++) {
@@ -515,7 +516,7 @@ _psyco_conn_parse_onoff(PyObject *pyval)
         rv = STATE_DEFAULT;
     }
     else if (PyUnicode_CheckExact(pyval) || Bytes_CheckExact(pyval)) {
-        if (!(pyval = psycopg_ensure_bytes(pyval))) {
+        if (!(pyval = psyco_ensure_bytes(pyval))) {
             goto exit;
         }
         if (0 == strcasecmp("default", Bytes_AS_STRING(pyval))) {
@@ -615,10 +616,7 @@ psyco_conn_set_session(connectionObject *self, PyObject *args, PyObject *kwargs)
 static PyObject *
 psyco_conn_autocommit_get(connectionObject *self)
 {
-    PyObject *ret;
-    ret = self->autocommit ? Py_True : Py_False;
-    Py_INCREF(ret);
-    return ret;
+    return PyBool_FromLong(self->autocommit);
 }
 
 BORROWED static PyObject *
@@ -824,6 +822,19 @@ psyco_conn_deferrable_set(connectionObject *self, PyObject *pyvalue)
     return 0;
 }
 
+/* psyco_get_native_connection - expose PGconn* as a Python capsule */
+
+#define psyco_get_native_connection_doc \
+"get_native_connection() -- Return the internal PGconn* as a Python Capsule."
+
+static PyObject *
+psyco_get_native_connection(connectionObject *self, PyObject *dummy)
+{
+    EXC_IF_CONN_CLOSED(self);
+
+    return PyCapsule_New(self->pgconn, "psycopg2.connection.native_connection", NULL);
+}
+
 
 /* set_client_encoding method - set client encoding */
 
@@ -855,7 +866,7 @@ psyco_conn_set_client_encoding(connectionObject *self, PyObject *args)
 "get_transaction_status() -- Get backend transaction status."
 
 static PyObject *
-psyco_conn_get_transaction_status(connectionObject *self)
+psyco_conn_get_transaction_status(connectionObject *self, PyObject *dummy)
 {
     return PyInt_FromLong((long)PQtransactionStatus(self->pgconn));
 }
@@ -894,7 +905,7 @@ psyco_conn_get_parameter_status(connectionObject *self, PyObject *args)
 "get_dsn_parameters() -- Get effective connection parameters.\n\n"
 
 static PyObject *
-psyco_conn_get_dsn_parameters(connectionObject *self)
+psyco_conn_get_dsn_parameters(connectionObject *self, PyObject *dummy)
 {
 #if PG_VERSION_NUM >= 90300
     PyObject *res = NULL;
@@ -907,7 +918,7 @@ psyco_conn_get_dsn_parameters(connectionObject *self)
         goto exit;
     }
 
-    res = psycopg_dict_from_conninfo_options(options, /* include_password = */ 0);
+    res = psyco_dict_from_conninfo_options(options, /* include_password = */ 0);
 
 exit:
     PQconninfoFree(options);
@@ -987,20 +998,51 @@ psyco_conn_lobject(connectionObject *self, PyObject *args, PyObject *keywds)
 "get_backend_pid() -- Get backend process id."
 
 static PyObject *
-psyco_conn_get_backend_pid(connectionObject *self)
+psyco_conn_get_backend_pid(connectionObject *self, PyObject *dummy)
 {
     EXC_IF_CONN_CLOSED(self);
 
     return PyInt_FromLong((long)PQbackendPID(self->pgconn));
 }
 
+
+/* get info about the connection */
+
+#define psyco_conn_info_doc \
+"info -- Get connection info."
+
+static PyObject *
+psyco_conn_info_get(connectionObject *self)
+{
+    return PyObject_CallFunctionObjArgs(
+        (PyObject *)&connInfoType, (PyObject *)self, NULL);
+}
+
+
+/* return the pointer to the PGconn structure */
+
+#define psyco_conn_pgconn_ptr_doc \
+"pgconn_ptr -- Get the PGconn structure pointer."
+
+static PyObject *
+psyco_conn_pgconn_ptr_get(connectionObject *self)
+{
+    if (self->pgconn) {
+        return PyLong_FromVoidPtr((void *)self->pgconn);
+    }
+    else {
+        Py_RETURN_NONE;
+    }
+}
+
+
 /* reset the currect connection */
 
 #define psyco_conn_reset_doc \
 "reset() -- Reset current connection to defaults."
 
 static PyObject *
-psyco_conn_reset(connectionObject *self)
+psyco_conn_reset(connectionObject *self, PyObject *dummy)
 {
     int res;
 
@@ -1010,7 +1052,7 @@ psyco_conn_reset(connectionObject *self)
     if (pq_reset(self) < 0)
         return NULL;
 
-    res = conn_setup(self, self->pgconn);
+    res = conn_setup(self);
     if (res < 0)
         return NULL;
 
@@ -1031,7 +1073,7 @@ psyco_conn_get_exception(PyObject *self, void *closure)
 "poll() -> int -- Advance the connection or query process without blocking."
 
 static PyObject *
-psyco_conn_poll(connectionObject *self)
+psyco_conn_poll(connectionObject *self, PyObject *dummy)
 {
     int res;
 
@@ -1051,7 +1093,7 @@ psyco_conn_poll(connectionObject *self)
 "fileno() -> int -- Return file descriptor associated to database connection."
 
 static PyObject *
-psyco_conn_fileno(connectionObject *self)
+psyco_conn_fileno(connectionObject *self, PyObject *dummy)
 {
     long int socket;
 
@@ -1068,29 +1110,25 @@ psyco_conn_fileno(connectionObject *self)
  "executing an asynchronous operation."
 
 static PyObject *
-psyco_conn_isexecuting(connectionObject *self)
+psyco_conn_isexecuting(connectionObject *self, PyObject *dummy)
 {
     /* synchronous connections will always return False */
     if (self->async == 0) {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
 
     /* check if the connection is still being built */
     if (self->status != CONN_STATUS_READY) {
-        Py_INCREF(Py_True);
-        return Py_True;
+        Py_RETURN_TRUE;
     }
 
     /* check if there is a query being executed */
     if (self->async_cursor != NULL) {
-        Py_INCREF(Py_True);
-        return Py_True;
+        Py_RETURN_TRUE;
     }
 
     /* otherwise it's not executing */
-    Py_INCREF(Py_False);
-    return Py_False;
+    Py_RETURN_FALSE;
 }
 
 
@@ -1098,7 +1136,7 @@ psyco_conn_isexecuting(connectionObject *self)
 "cancel() -- cancel the current operation"
 
 static PyObject *
-psyco_conn_cancel(connectionObject *self)
+psyco_conn_cancel(connectionObject *self, PyObject *dummy)
 {
     char errbuf[256];
 
@@ -1179,6 +1217,8 @@ static struct PyMethodDef connectionObject_methods[] = {
      METH_NOARGS, psyco_conn_isexecuting_doc},
     {"cancel", (PyCFunction)psyco_conn_cancel,
      METH_NOARGS, psyco_conn_cancel_doc},
+    {"get_native_connection", (PyCFunction)psyco_get_native_connection,
+     METH_NOARGS, psyco_get_native_connection_doc},
     {NULL}
 };
 
@@ -1245,6 +1285,12 @@ static struct PyGetSetDef connectionObject_getsets[] = {
         (getter)psyco_conn_deferrable_get,
         (setter)psyco_conn_deferrable_set,
         psyco_conn_deferrable_doc },
+    { "info",
+        (getter)psyco_conn_info_get, NULL,
+        psyco_conn_info_doc },
+    { "pgconn_ptr",
+        (getter)psyco_conn_pgconn_ptr_get, NULL,
+        psyco_conn_pgconn_ptr_doc },
     {NULL}
 };
 #undef EXCEPTION_GETTER
@@ -1268,7 +1314,7 @@ obscure_password(connectionObject *conn)
         return 0;
     }
 
-    if (!(d = psycopg_dict_from_conninfo_options(
+    if (!(d = psyco_dict_from_conninfo_options(
             options, /* include_password = */ 1))) {
         goto exit;
     }
@@ -1281,12 +1327,12 @@ obscure_password(connectionObject *conn)
     /* scrub the password and put back the connection string together */
     if (!(v = Text_FromUTF8("xxx"))) { goto exit; }
     if (0 > PyDict_SetItemString(d, "password", v)) { goto exit; }
-    if (!(dsn = psycopg_make_dsn(Py_None, d))) { goto exit; }
-    if (!(dsn = psycopg_ensure_bytes(dsn))) { goto exit; }
+    if (!(dsn = psyco_make_dsn(Py_None, d))) { goto exit; }
+    if (!(dsn = psyco_ensure_bytes(dsn))) { goto exit; }
 
     /* Replace the connection string on the connection object */
     tmp = conn->dsn;
-    psycopg_strdup(&conn->dsn, Bytes_AS_STRING(dsn), -1);
+    psyco_strdup(&conn->dsn, Bytes_AS_STRING(dsn), -1);
     PyMem_Free(tmp);
 
     rv = 0;
@@ -1310,7 +1356,7 @@ connection_setup(connectionObject *self, const char *dsn, long int async)
             self, async, Py_REFCNT(self)
       );
 
-    if (0 > psycopg_strdup(&self->dsn, dsn, -1)) { goto exit; }
+    if (0 > psyco_strdup(&self->dsn, dsn, -1)) { goto exit; }
     if (!(self->notice_list = PyList_New(0))) { goto exit; }
     if (!(self->notifies = PyList_New(0))) { goto exit; }
     self->async = async;
@@ -1321,6 +1367,10 @@ connection_setup(connectionObject *self, const char *dsn, long int async)
     self->isolevel = ISOLATION_LEVEL_DEFAULT;
     self->readonly = STATE_DEFAULT;
     self->deferrable = STATE_DEFAULT;
+#ifdef CONN_CHECK_PID
+    self->procpid = getpid();
+#endif
+
     /* other fields have been zeroed by tp_alloc */
 
     pthread_mutex_init(&(self->lock), NULL);
@@ -1374,7 +1424,15 @@ connection_dealloc(PyObject* obj)
      * resulting in a double-free segfault (ticket #166). */
     PyObject_GC_UnTrack(self);
 
-    conn_close(self);
+    /* close the connection only if this is the same process it was created
+     * into, otherwise using multiprocessing we may close the connection
+     * belonging to another process. */
+#ifdef CONN_CHECK_PID
+    if (self->procpid == getpid())
+#endif
+    {
+        conn_close(self);
+    }
 
     if (self->weakreflist) {
         PyObject_ClearWeakRefs(obj);
@@ -1384,8 +1442,9 @@ connection_dealloc(PyObject* obj)
 
     PyMem_Free(self->dsn);
     PyMem_Free(self->encoding);
-    if (self->critical) free(self->critical);
+    if (self->error) free(self->error);
     if (self->cancel) PQfreeCancel(self->cancel);
+    PQclear(self->pgres);
 
     connection_clear(self);
 
