@@ -1,6 +1,6 @@
 /* pqpath.h - definitions for pqpath.c
  *
- * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
@@ -28,48 +28,47 @@
 
 #include "psycopg/cursor.h"
 #include "psycopg/connection.h"
+#include "psycopg/replication_cursor.h"
+#include "psycopg/replication_message.h"
 
 /* macro to clean the pg result */
 #define CLEARPGRES(pgres)   do { PQclear(pgres); pgres = NULL; } while (0)
 
 /* exported functions */
-HIDDEN PGresult *pq_get_last_result(connectionObject *conn);
 RAISES_NEG HIDDEN int pq_fetch(cursorObject *curs, int no_result);
 RAISES_NEG HIDDEN int pq_execute(cursorObject *curs, const char *query,
-                                 int async, int no_result);
+                                 int async, int no_result, int no_begin);
 HIDDEN int pq_send_query(connectionObject *conn, const char *query);
-HIDDEN int pq_begin_locked(connectionObject *conn, PGresult **pgres,
-                           char **error, PyThreadState **tstate);
+HIDDEN int pq_begin_locked(connectionObject *conn, PyThreadState **tstate);
 HIDDEN int pq_commit(connectionObject *conn);
-RAISES_NEG HIDDEN int pq_abort_locked(connectionObject *conn, PGresult **pgres,
-                           char **error, PyThreadState **tstate);
+RAISES_NEG HIDDEN int pq_abort_locked(connectionObject *conn,
+                           PyThreadState **tstate);
 RAISES_NEG HIDDEN int pq_abort(connectionObject *conn);
-HIDDEN int pq_reset_locked(connectionObject *conn, PGresult **pgres,
-                            char **error, PyThreadState **tstate);
+HIDDEN int pq_reset_locked(connectionObject *conn, PyThreadState **tstate);
 RAISES_NEG HIDDEN int pq_reset(connectionObject *conn);
 HIDDEN char *pq_get_guc_locked(connectionObject *conn, const char *param,
-                               PGresult **pgres,
-                               char **error, PyThreadState **tstate);
+                               PyThreadState **tstate);
 HIDDEN int pq_set_guc_locked(connectionObject *conn, const char *param,
-                             const char *value, PGresult **pgres,
-                             char **error, PyThreadState **tstate);
+                             const char *value, PyThreadState **tstate);
 HIDDEN int pq_tpc_command_locked(connectionObject *conn,
                                  const char *cmd, const char *tid,
-                                 PGresult **pgres, char **error,
                                  PyThreadState **tstate);
-HIDDEN int pq_is_busy(connectionObject *conn);
-HIDDEN int pq_is_busy_locked(connectionObject *conn);
+RAISES_NEG HIDDEN int pq_get_result_async(connectionObject *conn);
 HIDDEN int pq_flush(connectionObject *conn);
 HIDDEN void pq_clear_async(connectionObject *conn);
 RAISES_NEG HIDDEN int pq_set_non_blocking(connectionObject *conn, int arg);
 
 HIDDEN void pq_set_critical(connectionObject *conn, const char *msg);
 
-HIDDEN int pq_execute_command_locked(connectionObject *conn,
-                                     const char *query,
-                                     PGresult **pgres, char **error,
+HIDDEN int pq_execute_command_locked(connectionObject *conn, const char *query,
                                      PyThreadState **tstate);
-RAISES HIDDEN void pq_complete_error(connectionObject *conn, PGresult **pgres,
-                              char **error);
+RAISES HIDDEN void pq_complete_error(connectionObject *conn);
+
+/* replication protocol support */
+HIDDEN int pq_copy_both(replicationCursorObject *repl, PyObject *consumer,
+                        double keepalive_interval);
+HIDDEN int pq_read_replication_message(replicationCursorObject *repl,
+                                       replicationMessageObject **msg);
+HIDDEN int pq_send_replication_feedback(replicationCursorObject *repl, int reply_requested);
 
 #endif /* !defined(PSYCOPG_PQPATH_H) */
