@@ -2,7 +2,7 @@
 #
 # types_basic.py - tests for basic types conversions
 #
-# Copyright (C) 2004-2010 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2004-2019 Federico Di Gregorio  <fog@debian.org>
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -22,14 +22,18 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
+import string
+import ctypes
 import decimal
+import datetime
+import platform
 
-import sys
-from functools import wraps
-import testutils
-from testutils import unittest, ConnectingTestCase, decorate_all_tests
+from . import testutils
+import unittest
+from .testutils import PY2, long, text_type, ConnectingTestCase
 
 import psycopg2
+from psycopg2.extensions import AsIs, adapt, register_adapter
 
 
 class TypesBasicTests(ConnectingTestCase):
@@ -53,8 +57,8 @@ class TypesBasicTests(ConnectingTestCase):
     def testNumber(self):
         s = self.execute("SELECT %s AS foo", (1971,))
         self.failUnless(s == 1971, "wrong integer quoting: " + str(s))
-        s = self.execute("SELECT %s AS foo", (1971L,))
-        self.failUnless(s == 1971L, "wrong integer quoting: " + str(s))
+        s = self.execute("SELECT %s AS foo", (long(1971),))
+        self.failUnless(s == long(1971), "wrong integer quoting: " + str(s))
 
     def testBoolean(self):
         x = self.execute("SELECT %s as foo", (False,))
@@ -104,7 +108,7 @@ class TypesBasicTests(ConnectingTestCase):
         self.failUnless(str(s) == "-inf", "wrong float quoting: " + str(s))
 
     def testBinary(self):
-        if sys.version_info[0] < 3:
+        if PY2:
             s = ''.join([chr(x) for x in range(256)])
             b = psycopg2.Binary(s)
             buf = self.execute("SELECT %s::bytea AS foo", (b,))
@@ -122,7 +126,7 @@ class TypesBasicTests(ConnectingTestCase):
 
     def testBinaryEmptyString(self):
         # test to make sure an empty Binary is converted to an empty string
-        if sys.version_info[0] < 3:
+        if PY2:
             b = psycopg2.Binary('')
             self.assertEqual(str(b), "''::bytea")
         else:
@@ -132,7 +136,7 @@ class TypesBasicTests(ConnectingTestCase):
     def testBinaryRoundTrip(self):
         # test to make sure buffers returned by psycopg2 are
         # understood by execute:
-        if sys.version_info[0] < 3:
+        if PY2:
             s = ''.join([chr(x) for x in range(256)])
             buf = self.execute("SELECT %s::bytea AS foo", (psycopg2.Binary(s),))
             buf2 = self.execute("SELECT %s::bytea AS foo", (buf,))
@@ -151,7 +155,6 @@ class TypesBasicTests(ConnectingTestCase):
 
     def testEmptyArrayRegression(self):
         # ticket #42
-        import datetime
         curs = self.conn.cursor()
         curs.execute(
             "create table array_test "
@@ -206,6 +209,31 @@ class TypesBasicTests(ConnectingTestCase):
             self.assertRaises(psycopg2.DataError,
                 psycopg2.extensions.STRINGARRAY, s.encode('utf8'), curs)
 
+    def testTextArray(self):
+        curs = self.conn.cursor()
+        curs.execute("select '{a,b,c}'::text[]")
+        x = curs.fetchone()[0]
+        self.assert_(isinstance(x[0], str))
+        self.assertEqual(x, ['a', 'b', 'c'])
+
+    def testUnicodeArray(self):
+        psycopg2.extensions.register_type(
+            psycopg2.extensions.UNICODEARRAY, self.conn)
+        curs = self.conn.cursor()
+        curs.execute("select '{a,b,c}'::text[]")
+        x = curs.fetchone()[0]
+        self.assert_(isinstance(x[0], text_type))
+        self.assertEqual(x, [u'a', u'b', u'c'])
+
+    def testBytesArray(self):
+        psycopg2.extensions.register_type(
+            psycopg2.extensions.BYTESARRAY, self.conn)
+        curs = self.conn.cursor()
+        curs.execute("select '{a,b,c}'::text[]")
+        x = curs.fetchone()[0]
+        self.assert_(isinstance(x[0], bytes))
+        self.assertEqual(x, [b'a', b'b', b'c'])
+
     @testutils.skip_before_postgres(8, 2)
     def testArrayOfNulls(self):
         curs = self.conn.cursor()
@@ -234,9 +262,9 @@ class TypesBasicTests(ConnectingTestCase):
         curs.execute("insert into na (textaa) values (%s)", ([['a', None]],))
         curs.execute("insert into na (textaa) values (%s)", ([[None, None]],))
 
-        curs.execute("insert into na (intaa) values (%s)",  ([[None]],))
+        curs.execute("insert into na (intaa) values (%s)", ([[None]],))
         curs.execute("insert into na (intaa) values (%s)", ([[42, None]],))
-        curs.execute("insert into na (intaa) values (%s)",  ([[None, None]],))
+        curs.execute("insert into na (intaa) values (%s)", ([[None, None]],))
 
         curs.execute("insert into na (boolaa) values (%s)", ([[None]],))
         curs.execute("insert into na (boolaa) values (%s)", ([[True, None]],))
@@ -294,12 +322,11 @@ class TypesBasicTests(ConnectingTestCase):
         o2 = self.execute("select %s;", (o1,))
         self.assertEqual(memoryview, type(o2[0]))
 
-    @testutils.skip_before_python(2, 6)
     def testAdaptBytearray(self):
         o1 = bytearray(range(256))
         o2 = self.execute("select %s;", (o1,))
 
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(buffer, type(o2))
         else:
             self.assertEqual(memoryview, type(o2))
@@ -313,16 +340,15 @@ class TypesBasicTests(ConnectingTestCase):
         o2 = self.execute("select %s;", (o1,))
 
         self.assertEqual(len(o2), 0)
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(buffer, type(o2))
         else:
             self.assertEqual(memoryview, type(o2))
 
-    @testutils.skip_before_python(2, 7)
     def testAdaptMemoryview(self):
         o1 = memoryview(bytearray(range(256)))
         o2 = self.execute("select %s;", (o1,))
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(buffer, type(o2))
         else:
             self.assertEqual(memoryview, type(o2))
@@ -330,7 +356,7 @@ class TypesBasicTests(ConnectingTestCase):
         # Test with an empty buffer
         o1 = memoryview(bytearray([]))
         o2 = self.execute("select %s;", (o1,))
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(buffer, type(o2))
         else:
             self.assertEqual(memoryview, type(o2))
@@ -349,7 +375,7 @@ class TypesBasicTests(ConnectingTestCase):
         self.assertEqual(1, f1)
         i1 = self.execute("select -%s;", (-1,))
         self.assertEqual(1, i1)
-        l1 = self.execute("select -%s;", (-1L,))
+        l1 = self.execute("select -%s;", (long(-1),))
         self.assertEqual(1, l1)
 
     def testGenericArray(self):
@@ -383,11 +409,21 @@ class TypesBasicTests(ConnectingTestCase):
         a = self.execute("select '{10:20:30:40:50:60}'::macaddr[]")
         self.assertEqual(a, ['10:20:30:40:50:60'])
 
+    @testutils.skip_before_python(3, 4)
+    def testIntEnum(self):
+        from enum import IntEnum
+
+        class Color(IntEnum):
+            RED = 1
+            GREEN = 2
+            BLUE = 4
+
+        a = self.execute("select %s", (Color.GREEN,))
+        self.assertEqual(a, Color.GREEN)
+
 
 class AdaptSubclassTest(unittest.TestCase):
     def test_adapt_subtype(self):
-        from psycopg2.extensions import adapt
-
         class Sub(str):
             pass
         s1 = "hel'lo"
@@ -395,8 +431,6 @@ class AdaptSubclassTest(unittest.TestCase):
         self.assertEqual(adapt(s1).getquoted(), adapt(s2).getquoted())
 
     def test_adapt_most_specific(self):
-        from psycopg2.extensions import adapt, register_adapter, AsIs
-
         class A(object):
             pass
 
@@ -416,8 +450,6 @@ class AdaptSubclassTest(unittest.TestCase):
 
     @testutils.skip_from_python(3)
     def test_no_mro_no_joy(self):
-        from psycopg2.extensions import adapt, register_adapter, AsIs
-
         class A:
             pass
 
@@ -432,8 +464,6 @@ class AdaptSubclassTest(unittest.TestCase):
 
     @testutils.skip_before_python(3)
     def test_adapt_subtype_3(self):
-        from psycopg2.extensions import adapt, register_adapter, AsIs
-
         class A:
             pass
 
@@ -447,9 +477,6 @@ class AdaptSubclassTest(unittest.TestCase):
             del psycopg2.extensions.adapters[A, psycopg2.extensions.ISQLQuote]
 
     def test_conform_subclass_precedence(self):
-
-        import psycopg2.extensions as ext
-
         class foo(tuple):
             def __conform__(self, proto):
                 return self
@@ -457,25 +484,19 @@ class AdaptSubclassTest(unittest.TestCase):
             def getquoted(self):
                 return 'bar'
 
-        self.assertEqual(ext.adapt(foo((1, 2, 3))).getquoted(), 'bar')
+        self.assertEqual(adapt(foo((1, 2, 3))).getquoted(), 'bar')
 
 
+@unittest.skipIf(
+    platform.system() == 'Windows',
+    "Not testing because we are useless with ctypes on Windows")
 class ByteaParserTest(unittest.TestCase):
     """Unit test for our bytea format parser."""
     def setUp(self):
-        try:
-            self._cast = self._import_cast()
-        except Exception as e:
-            self._cast = None
-            self._exc = e
+        self._cast = self._import_cast()
 
     def _import_cast(self):
-        """Use ctypes to access the C function.
-
-        Raise any sort of error: we just support this where ctypes works as
-        expected.
-        """
-        import ctypes
+        """Use ctypes to access the C function."""
         lib = ctypes.pydll.LoadLibrary(psycopg2._psycopg.__file__)
         cast = lib.typecast_BINARY_cast
         cast.argtypes = [ctypes.c_char_p, ctypes.c_size_t, ctypes.py_object]
@@ -490,7 +511,7 @@ class ByteaParserTest(unittest.TestCase):
         if rv is None:
             return None
 
-        if sys.version_info[0] < 3:
+        if PY2:
             return str(rv)
         else:
             return rv.tobytes()
@@ -514,7 +535,7 @@ class ByteaParserTest(unittest.TestCase):
             buf = buf.upper()
         buf = '\\x' + buf
         rv = self.cast(buf.encode('utf8'))
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(rv, ''.join(map(chr, range(256))))
         else:
             self.assertEqual(rv, bytes(range(256)))
@@ -525,19 +546,18 @@ class ByteaParserTest(unittest.TestCase):
     def test_full_escaped_octal(self):
         buf = ''.join(("\\%03o" % i) for i in range(256))
         rv = self.cast(buf.encode('utf8'))
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(rv, ''.join(map(chr, range(256))))
         else:
             self.assertEqual(rv, bytes(range(256)))
 
     def test_escaped_mixed(self):
-        import string
         buf = ''.join(("\\%03o" % i) for i in range(32))
         buf += string.ascii_letters
         buf += ''.join('\\' + c for c in string.ascii_letters)
         buf += '\\\\'
         rv = self.cast(buf.encode('utf8'))
-        if sys.version_info[0] < 3:
+        if PY2:
             tgt = ''.join(map(chr, range(32))) \
                 + string.ascii_letters * 2 + '\\'
         else:
@@ -547,22 +567,9 @@ class ByteaParserTest(unittest.TestCase):
         self.assertEqual(rv, tgt)
 
 
-def skip_if_cant_cast(f):
-    @wraps(f)
-    def skip_if_cant_cast_(self, *args, **kwargs):
-        if self._cast is None:
-            return self.skipTest("can't test bytea parser: %s - %s"
-                % (self._exc.__class__.__name__, self._exc))
-
-        return f(self, *args, **kwargs)
-
-    return skip_if_cant_cast_
-
-decorate_all_tests(ByteaParserTest, skip_if_cant_cast)
-
-
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
